#include <check.h>
#include <hitmap.h>


/***************************************************************************************************
 ***************************************************************************************************
 *
 *                      TESTS: hit_layout_plug_layDimBlocksWeighted_Sig
 *
***************************************************************************************************
***************************************************************************************************/

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigTestValid1
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigTestValid)
{
    Load_ratios ratio;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 40;
    ratio.ratios[2] = 60;
    ratio.ratios[3] = 71;
    ratio.ratios[4] = 100;

    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)

    HitSig hs1;
    int p0_min = ratio.ratios[0];
    int p1_min = ratio.ratios[1] * card / ratio.ratios[ratio.num_procs];
    int p2_min = ratio.ratios[2] * card / ratio.ratios[ratio.num_procs];
    int p3_min = ratio.ratios[3] * card / ratio.ratios[ratio.num_procs];

    int ret = hit_layout_plug_layDimBlocksWeighted_Sig(0, ratio.num_procs, card,
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);

    ck_assert_int_eq(ret, 1); /* Active */
    ck_assert_int_eq(p0_min, hs1.begin);
    ck_assert_int_eq(p1_min - 1, hs1.end);
    ck_assert_int_eq(signature_base.stride, hs1.stride);

    int i, count = 0, end = -1;
    for(i=0; i<ratio.num_procs; i++)
    {
       int ret = hit_layout_plug_layDimBlocksWeighted_Sig(i, ratio.num_procs, card,
            
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);
       ck_assert_int_eq(ret, 1); /*Active */
       ck_assert_int_eq(hs1.begin, end+ 1); /* All the signature has to be continous without any gap */

       end = hs1.end;
       count += hit_sigCard(hs1);

    }

    ck_assert_int_eq(signature_base.end, end); /* Check final signature */

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigTestValidMoreProcesses
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigTestValidMoreProcesses)
{
    Load_ratios ratio;

    ratio.num_procs = 8;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 10;
    ratio.ratios[2] = 12;
    ratio.ratios[3] = 14;
    ratio.ratios[4] = 16;
    ratio.ratios[5] = 18;
    ratio.ratios[6] = 20;
    ratio.ratios[7] = 60;
    ratio.ratios[8] = 100;

    HitSig hs1;
    HitSig signature_base = hit_sig(0, 28, 2);
    int card = hit_sigCard(signature_base); // 15  (0 to 14)

    int cardAccum = 0;
    int i;

    for (i = 0; i <= 7; i++)
    {
        int ret = hit_layout_plug_layDimBlocksWeighted_Sig(i, ratio.num_procs, card,
                                                      (float *)&ratio, signature_base, &hs1);
        cardAccum += hit_sigCard(hs1);
    }

    ck_assert_int_eq(cardAccum , card);

    int count = 0, end = -2;
    for(i=0; i<ratio.num_procs; i++)
    {
       int active = hit_layout_plug_layDimBlocksWeighted_Sig(i, ratio.num_procs, card,           
                                                  (float *)&ratio, signature_base, &hs1);
        if(active)
        {
            ck_assert_int_eq(hs1.begin, end + signature_base.stride); /* All the signature has to be continous without any gap */
            end = hs1.end;
            count += hit_sigCard(hs1);
        }

    }

    ck_assert_int_eq(signature_base.end, end); /* Check final signature */

    free(ratio.ratios);
}
END_TEST


/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigTestWithProcsNonActivated
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigTestWithProcsNonActivated)
{
    Load_ratios ratio;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.4;
    ratio.ratios[2] = 0.41;
    ratio.ratios[3] = 0.71;
    ratio.ratios[4] = 1;

    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)

    HitSig hs1;

    // PROC 1 -> Non active
    int ret = hit_layout_plug_layDimBlocksWeighted_Sig(1, ratio.num_procs, card,
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);

    ck_assert_int_eq(ret, 0); /* Non Active */

    ck_assert(hs1.begin == HIT_SIG_NULL.begin); /* Non Active */
    ck_assert(hs1.end == HIT_SIG_NULL.end); /* Non Active */
    ck_assert(hs1.stride == HIT_SIG_NULL.stride); /* Non Active */

    int i = 0, count = 0, end = -1;
    for(i=0; i<ratio.num_procs; i++)
    {
        if(i != 1)
        {
            ret = hit_layout_plug_layDimBlocksWeighted_Sig(i, ratio.num_procs, card,
            
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);
            ck_assert_int_eq(ret, 1); /*Active */
            ck_assert_int_eq(hs1.begin, end+ 1); /* All the signature has to be continous without any gap */

            end = hs1.end;
            count += hit_sigCard(hs1);
        }
    }

    ck_assert_int_eq(signature_base.end, end); /* Check final signature */

    // The sum of all proccesses card is equal to the original card
    ck_assert_int_eq(count, card);

    free(ratio.ratios);
}
END_TEST


/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigTestProcIdGreaterThanMax
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigTestProcIdGreaterThanMax)
{
    Load_ratios ratio;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.4;
    ratio.ratios[2] = 0.6;
    ratio.ratios[3] = 0.71;
    ratio.ratios[4] = 1;

    HitSig signature_base = hit_sig(0, 49, 1);

    HitSig hs1;

    int ret = hit_layout_plug_layDimBlocksWeighted_Sig(4, ratio.num_procs, 4,            
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);

    ck_assert_int_eq(ret, 0);
    ck_assert_int_eq(hs1.begin, HIT_SIG_NULL.begin); /* Non valid */
    ck_assert_int_eq(hs1.end, HIT_SIG_NULL.end); /* Non valid */
    ck_assert_int_eq(hs1.stride, HIT_SIG_NULL.stride); /* Non valid */

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigTestProcIdLowerThanMin
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigTestProcIdLowerThanMin)
{
    Load_ratios ratio;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.4;
    ratio.ratios[2] = 0.6;
    ratio.ratios[3] = 0.71;
    ratio.ratios[4] = 1;

    HitSig signature_base = hit_sig(0, 49, 1);

    HitSig hs1;

    int ret = hit_layout_plug_layDimBlocksWeighted_Sig(4, ratio.num_procs, -1,            
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);

    ck_assert_int_eq(ret, 0);
    ck_assert_int_eq(hs1.begin, HIT_SIG_NULL.begin); /* Non valid */
    ck_assert_int_eq(hs1.end, HIT_SIG_NULL.end); /* Non valid */
    ck_assert_int_eq(hs1.stride, HIT_SIG_NULL.stride); /* Non valid */

    free(ratio.ratios);
}
END_TEST


/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigTestBlocksCardLessOrEqualTo0
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigTestBlocksCardLessOrEqualTo0)
{
    Load_ratios ratio;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.4;
    ratio.ratios[2] = 0.6;
    ratio.ratios[3] = 0.71;
    ratio.ratios[4] = 1;

    HitSig signature_base = hit_sig(0, 49, 1);

    HitSig hs1;

    int ret = hit_layout_plug_layDimBlocksWeighted_Sig(0, 0, 1,            
                                                  (float *)&ratio, hit_sig(0, 49, 1), &hs1);

    ck_assert_int_eq(ret, 0);
    ck_assert_int_eq(hs1.begin, HIT_SIG_NULL.begin); /* Non valid */
    ck_assert_int_eq(hs1.end, HIT_SIG_NULL.end); /* Non valid */
    ck_assert_int_eq(hs1.stride, HIT_SIG_NULL.stride); /* Non valid */

    free(ratio.ratios);
}
END_TEST

/***************************************************************************************************
 ***************************************************************************************************
 *
 *                      TESTS: hit_layout_plug_layDimBlocksWeighted_SigInv
 *
***************************************************************************************************
***************************************************************************************************/

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestValid
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestValid)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int i,ret,j;
    HitSig hs1;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.3;
    ratio.ratios[2] = 0.5;
    ratio.ratios[3] = 0.7;
    ratio.ratios[4] = 1;

    float *index = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    index[0] = 0;
    index[1] = 0.3 * card;
    index[2] = 0.5 * card;
    index[3] = 0.7 * card;
    index[4] = card;

    for(i = 0; i < ratio.num_procs; i++)
    {
        for(j = (int)index[i]; j < (int)index[i + 1]; j++)
        {
            ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, ratio.num_procs, card,
                                                  (float *)&ratio, signature_base, j); 
            // Returns the real number of the proc
            ck_assert_int_eq(ret, i); /* Non Active */
        }
                                    
    }
    free(index);
    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestWithProcsNonActivated
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestWithProcsNonActivated)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int i,ret,j;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.3;
    ratio.ratios[2] = 0.305;
    ratio.ratios[3] = 0.7;
    ratio.ratios[4] = 0.71;
    ratio.ratios[5] = 1;

    float *index = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    index[0] = 0;
    index[1] = 0.3 * card;
    index[2] = 0.305 * card;
    index[3] = 0.7 * card;
    index[4] = 0.71 * card;
    index[5] = card;

    for(i = 0; i < ratio.num_procs; i++)
    {
        for(j = (int)index[i]; j < (int)index[i + 1]; j++)
        {
            ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, ratio.num_procs, card,
                                                  (float *)&ratio, signature_base, j); 
            // Returns the real number of the proc
            ck_assert_int_eq(ret, i);
        }
                                    
    }

    free(index);
    free(ratio.ratios);
}
END_TEST


/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestIndexGreaterThanLimitOfSignature
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestIndexGreaterThanLimitOfSignature)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.3;
    ratio.ratios[2] = 0.5;
    ratio.ratios[3] = 0.7;
    ratio.ratios[4] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, ratio.num_procs, card,
                                                  (float *)&ratio, signature_base, signature_base.end + 1); 
    // Returns HIT_RANK_NULL
    ck_assert_int_eq(ret, HIT_RANK_NULL);

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestIndexLowerThanLimitOfSignature
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestIndexLowerThanLimitOfSignature)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.3;
    ratio.ratios[2] = 0.5;
    ratio.ratios[3] = 0.7;
    ratio.ratios[4] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, ratio.num_procs, card,
                                                  (float *)&ratio, signature_base, signature_base.begin - 1); 
    // Returns HIT_RANK_NULL
    ck_assert_int_eq(ret, HIT_RANK_NULL);

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestBlocksCardLessOrEqualTo0
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestBlocksCardLessOrEqualTo0)
{
    Load_ratios ratio;

    ratio.num_procs = 4;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.4;
    ratio.ratios[2] = 0.6;
    ratio.ratios[3] = 0.71;
    ratio.ratios[4] = 1;

    HitSig signature_base = hit_sig(0, 49, 1);

    HitSig hs1;

    int ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, 1, 0,            
                                                  (float *)&ratio, hit_sig(0, 49, 1), 3);

    ck_assert_int_eq(ret, HIT_RANK_NULL);

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestLimitIndex
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestLimitIndex)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.37; // 18 (0.37 * 50)
    ratio.ratios[2] = 0.45;
    ratio.ratios[3] = 0.7;
    ratio.ratios[4] = 0.8;
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, 5, card,            
                                                  (float *)&ratio, hit_sig(0, 49, 1), 18);

    ck_assert_int_eq(ret, 1);

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_SigInvTestLimitIndexWithNonActivedProcessesBefore
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_SigInvTestLimitIndexWithNonActivedProcessesBefore)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.18;
    ratio.ratios[2] = 0;
    ratio.ratios[3] = 0;
    ratio.ratios[4] = 0.37; // 18 (0.37 * 50)
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_SigInv(0, 5, card,            
                                                  (float *)&ratio, hit_sig(0, 49, 1), 18);

    ck_assert_int_eq(ret, 4);

    free(ratio.ratios);
}
END_TEST

/***************************************************************************************************
 ***************************************************************************************************
 *
 *                      TESTS: hit_layout_plug_layDimBlocksWeighted_ranks
 *
***************************************************************************************************
***************************************************************************************************/

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_ranksTestValid
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_ranksTestValid)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.24;
    ratio.ratios[3] = 0.30; 
    ratio.ratios[4] = 0.35; 
    ratio.ratios[5] = 1;


    /* TOPO TO ACTIVE */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_TOPO_TO_ACTIVE, 3,ratio.num_procs, card,            
                                                  (float *)&ratio);
    ck_assert_int_eq(ret, 3);


    /* ACTIVE TO TOPO */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_ACTIVE_TO_TOPO, 3,ratio.num_procs, card,            
                                                   (float *)&ratio);
    ck_assert_int_eq(ret, 3);

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_ranksTestValidWithNonActivatedProcs
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_ranksTestValidWithNonActivatedProcs)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.24;
    ratio.ratios[3] = 0.24; 
    ratio.ratios[4] = 0.35; 
    ratio.ratios[5] = 1;


    /* TOPO TO ACTIVE */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_TOPO_TO_ACTIVE, 3,ratio.num_procs, card,            
                                                  (float *)&ratio);
    ck_assert_int_eq(ret, 2); // -1


    /* ACTIVE TO TOPO */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_ACTIVE_TO_TOPO, 3,ratio.num_procs, card,            
                                                   (float *)&ratio);
    ck_assert_int_eq(ret, 4); // +1

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_ranksTestTopotoActNonActiveProcess)
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_ranksTestTopotoActNonActiveProcess)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.24;
    ratio.ratios[3] = 0.24; 
    ratio.ratios[4] = 0.35; 
    ratio.ratios[5] = 1;

    /* TOPO TO ACTIVE */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_TOPO_TO_ACTIVE, 2,ratio.num_procs, card,            
                                                  (float *)&ratio);
    ck_assert_int_eq(ret, HIT_RANK_NULL); // -1

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_ranksTestActToTopoNonActiveProcess)
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_ranksTestActToTopoNonActiveProcess)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.24;
    ratio.ratios[3] = 0.24; 
    ratio.ratios[4] = 0.35; 
    ratio.ratios[5] = 1;

    /* TOPO TO ACTIVE */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_ACTIVE_TO_TOPO, 4,ratio.num_procs, card,            
                                                  (float *)&ratio);
    ck_assert_int_eq(ret, HIT_RANK_NULL); // -1

    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_rankranksTestEmptySignature)
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_ranksTestEmptySignature)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.24;
    ratio.ratios[3] = 0.24; 
    ratio.ratios[4] = 0.35; 
    ratio.ratios[5] = 1;

    /* TOPO TO ACTIVE */
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_TOPO_TO_ACTIVE, 4,ratio.num_procs, 0,            
                                                  (float *)&ratio);
    ck_assert_int_eq(ret, HIT_RANK_NULL); // -1

    /* ACTIVE TO TOPO*/
    ret = hit_layout_plug_layDimBlocksWeighted_ranks(HIT_LAY_RANKS_ACTIVE_TO_TOPO, 4,ratio.num_procs, 0,            
                                                  (float *)&ratio);
    ck_assert_int_eq(ret, HIT_RANK_NULL); // -1

    free(ratio.ratios);
}
END_TEST

/***************************************************************************************************
 ***************************************************************************************************
 *
 *                      TESTS: hit_layout_plug_layDimBlocksWeighted_maxCard
 *
***************************************************************************************************
***************************************************************************************************/

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_maxCardTestValid
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_maxCardTestValid)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.40; //Max card -> proc 2 with 12 (32 - 20)
    ratio.ratios[3] = 0.65; 
    ratio.ratios[4] = 0.85; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_maxCard( ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 12); // 12


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_maxCardTestSameMaxWeightsButDifferentCards
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_maxCardTestSameMaxWeightsButDifferentCards)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.31; //15
    ratio.ratios[2] = 0.43; 
    ratio.ratios[3] = 0.59; 
    ratio.ratios[4] = 0.69; //16
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_maxCard( ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 16); // 16


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_maxCardTestMinorWeightWithMaxCard
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_maxCardTestMinorWeightWithMaxCard)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 24, 1);
    int card = hit_sigCard(signature_base); // 25  (0 to 24)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.31; //card 7 (weight = 0.31)
    ratio.ratios[2] = 0.43; 
    ratio.ratios[3] = 0.59; 
    ratio.ratios[4] = 0.71; //card 8 (weight = 0.29)
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_maxCard( ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 8); // 8


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_maxCardTestBlocksCardEmptyOrLessThan0
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_maxCardTestBlocksCardEmptyOrLessThan0)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.30;
    ratio.ratios[2] = 0.43; 
    ratio.ratios[3] = 0.59; 
    ratio.ratios[4] = 0.60; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_maxCard( ratio.num_procs, 0, (float *)&ratio);
    ck_assert_int_eq(ret, 0); // 0 => blocksCard is 0


    free(ratio.ratios);
}
END_TEST

/***************************************************************************************************
 ***************************************************************************************************
 *
 *                      TESTS: hit_layout_plug_layDimBlocksWeighted_minCard
 *
***************************************************************************************************
***************************************************************************************************/

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_minCardTestValid
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_minCardTestValid)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20;
    ratio.ratios[2] = 0.40;
    ratio.ratios[3] = 0.65; 
    ratio.ratios[4] = 0.85; //Min card proc 4 with 8 (42 - 50)
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_minCard( ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 8); // 8


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_minCardTestSameMinWeightsButDifferentCards
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_minCardTestSameMinWeightsButDifferentCards)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.21; // 8 (0.15)
    ratio.ratios[2] = 0.36; // 7 (0.15)
    ratio.ratios[3] = 0.51; 
    ratio.ratios[4] = 0.80; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_minCard( ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 7); // 7


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_minCardTestMinorWeightWithMaxCard
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_minCardTestMinorWeightWithMaxCard)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 24, 1);
    int card = hit_sigCard(signature_base); // 25  (0 to 24)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.35; //card 2 (weight = 0.05)
    ratio.ratios[2] = 0.40; 
    ratio.ratios[3] = 0.60; //card 1 (weight = 0.07)
    ratio.ratios[4] = 0.67; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_minCard( ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 1); // 1


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_minCardTestBlocksCardEmptyOrLessThan0
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_minCardTestBlocksCardEmptyOrLessThan0)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.30;
    ratio.ratios[2] = 0.43; 
    ratio.ratios[3] = 0.59; 
    ratio.ratios[4] = 0.60; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_minCard( ratio.num_procs, 0, (float *)&ratio);
    ck_assert_int_eq(ret, 0); // 0 => blocksCard is 0


    free(ratio.ratios);
}
END_TEST

/***************************************************************************************************
 ***************************************************************************************************
 *
 *                      TESTS: hit_layout_plug_layDimBlocksWeighted_numActives
 *
***************************************************************************************************
***************************************************************************************************/

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_numActivesTestValid
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_numActivesTestValid)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20; //Non Actived
    ratio.ratios[2] = 0.20;
    ratio.ratios[3] = 0.65; 
    ratio.ratios[4] = 0.85; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_numActives(ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 4); // 4


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_numActivesTestAllZeros
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_numActivesTestAllZeros)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0; //Non Actived
    ratio.ratios[2] = 0;
    ratio.ratios[3] = 0; 
    ratio.ratios[4] = 0; 
    ratio.ratios[5] = 0;

    ret = hit_layout_plug_layDimBlocksWeighted_numActives(ratio.num_procs, card, (float *)&ratio);
    ck_assert_int_eq(ret, 0); // 0


    free(ratio.ratios);
}
END_TEST

/**
 * TEST
 * hit_layout_plug_layDimBlocksWeighted_numActivesTestCardEmptyOrLessThan0
 */
START_TEST(hit_layout_plug_layDimBlocksWeighted_numActivesTestBlocksCardEmptyOrLessThan0)
{
    Load_ratios ratio;
    HitSig signature_base = hit_sig(0, 49, 1);
    int card = hit_sigCard(signature_base); // 50  (0 to 49)
    int ret;
    HitSig hs1;

    ratio.num_procs = 5;
    ratio.ratios = (float *)malloc(sizeof(float) * (ratio.num_procs + 1));

    ratio.ratios[0] = 0;
    ratio.ratios[1] = 0.20; //Non Actived
    ratio.ratios[2] = 0.20;
    ratio.ratios[3] = 0.65; 
    ratio.ratios[4] = 0.85; 
    ratio.ratios[5] = 1;

    ret = hit_layout_plug_layDimBlocksWeighted_numActives(ratio.num_procs, 0, (float *)&ratio);
    ck_assert_int_eq(ret, 0); // 0


    free(ratio.ratios);
}
END_TEST

  
/***************************************************************************************************
 ***************************************************************************************************
 *
 *                                              MAIN
 *
***************************************************************************************************
***************************************************************************************************/


int main(void)
{
    Suite *s_main_suite_weightedLays = suite_create("Weighted Layouts Hitmap plug-in Tests");

    TCase *tc_hit_layout_plug_layDimBlocksWeighted_SigTest = tcase_create("hit_layout_plug_layDimBlocksWeighted_SigTest");
    TCase *tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest = tcase_create("hit_layout_plug_layDimBlocksWeighted_SigInvTest");
    TCase *tc_hit_layout_plug_layDimBlocksWeighted_ranksTest = tcase_create("hit_layout_plug_layDimBlocksWeighted_ranksTest");
    TCase *tc_hit_layout_plug_layDimBlocksWeighted_maxCardTest = tcase_create("hit_layout_plug_layDimBlocksWeighted_maxCardTest");
    TCase *tc_hit_layout_plug_layDimBlocksWeighted_minCardTest = tcase_create("hit_layout_plug_layDimBlocksWeighted_minCardTest");
    TCase *tc_hit_layout_plug_layDimBlocksWeighted_numActivesTest = tcase_create("hit_layout_plug_layDimBlocksWeighted_numActivesTest");

    SRunner *sr = srunner_create(s_main_suite_weightedLays);
    int nf;

    /* Add the test cases to main test suite */
    suite_add_tcase(s_main_suite_weightedLays, tc_hit_layout_plug_layDimBlocksWeighted_SigTest);
    suite_add_tcase(s_main_suite_weightedLays, tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest);
    suite_add_tcase(s_main_suite_weightedLays, tc_hit_layout_plug_layDimBlocksWeighted_ranksTest);
    suite_add_tcase(s_main_suite_weightedLays, tc_hit_layout_plug_layDimBlocksWeighted_maxCardTest);
    suite_add_tcase(s_main_suite_weightedLays, tc_hit_layout_plug_layDimBlocksWeighted_minCardTest);
    suite_add_tcase(s_main_suite_weightedLays, tc_hit_layout_plug_layDimBlocksWeighted_numActivesTest);


    /* Add the test to the test case tc_hit_layout_plug_layDimBlocksWeighted_SigTest */
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigTest, hit_layout_plug_layDimBlocksWeighted_SigTestValid);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigTest, hit_layout_plug_layDimBlocksWeighted_SigTestValidMoreProcesses);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigTest, hit_layout_plug_layDimBlocksWeighted_SigTestWithProcsNonActivated);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigTest, hit_layout_plug_layDimBlocksWeighted_SigTestProcIdGreaterThanMax);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigTest, hit_layout_plug_layDimBlocksWeighted_SigTestProcIdLowerThanMin);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigTest, hit_layout_plug_layDimBlocksWeighted_SigTestBlocksCardLessOrEqualTo0);

    /* Add the test to the test case tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest */
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestValid);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestWithProcsNonActivated);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestIndexGreaterThanLimitOfSignature);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestIndexLowerThanLimitOfSignature);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestBlocksCardLessOrEqualTo0);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestLimitIndex);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_SigInvTest, hit_layout_plug_layDimBlocksWeighted_SigInvTestLimitIndexWithNonActivedProcessesBefore);

    /* Add the test to the test case tc_hit_layout_plug_layDimBlocksWeighted_ranksTest */
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_ranksTest, hit_layout_plug_layDimBlocksWeighted_ranksTestValid);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_ranksTest, hit_layout_plug_layDimBlocksWeighted_ranksTestValidWithNonActivatedProcs);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_ranksTest, hit_layout_plug_layDimBlocksWeighted_ranksTestActToTopoNonActiveProcess);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_ranksTest, hit_layout_plug_layDimBlocksWeighted_ranksTestTopotoActNonActiveProcess);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_ranksTest, hit_layout_plug_layDimBlocksWeighted_ranksTestEmptySignature);

    /* Add the test to the test case tc_hit_layout_plug_layDimBlocksWeighted_maxCardTests */
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_maxCardTest, hit_layout_plug_layDimBlocksWeighted_maxCardTestValid);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_maxCardTest, hit_layout_plug_layDimBlocksWeighted_maxCardTestSameMaxWeightsButDifferentCards);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_maxCardTest, hit_layout_plug_layDimBlocksWeighted_maxCardTestMinorWeightWithMaxCard);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_maxCardTest, hit_layout_plug_layDimBlocksWeighted_maxCardTestBlocksCardEmptyOrLessThan0);

    /* Add the test to the test case tc_hit_layout_plug_layDimBlocksWeighted_minCardTests */
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_minCardTest, hit_layout_plug_layDimBlocksWeighted_minCardTestValid);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_minCardTest, hit_layout_plug_layDimBlocksWeighted_minCardTestSameMinWeightsButDifferentCards);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_minCardTest, hit_layout_plug_layDimBlocksWeighted_minCardTestMinorWeightWithMaxCard);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_minCardTest, hit_layout_plug_layDimBlocksWeighted_minCardTestBlocksCardEmptyOrLessThan0);

    /* Add the test to the test case tc_hit_layout_plug_layDimBlocksWeighted_numActivesTests */
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_numActivesTest, hit_layout_plug_layDimBlocksWeighted_numActivesTestValid);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_numActivesTest, hit_layout_plug_layDimBlocksWeighted_numActivesTestAllZeros);
    tcase_add_test(tc_hit_layout_plug_layDimBlocksWeighted_numActivesTest, hit_layout_plug_layDimBlocksWeighted_numActivesTestBlocksCardEmptyOrLessThan0);


    /* Run all tests */
    srunner_run_all(sr, CK_VERBOSE);
    nf = srunner_ntests_failed(sr);
    srunner_free(sr);

    return nf == 0 ? 0 : 1;
}
