#include <hitmap.h>
#include <stdio.h>
#include <stdlib.h>

hit_tileNewType(float);

int main(int argc, char *argv[])
{
    int arraySize = 50;
    int i;
    HitTopology globalTopology;

    HitShape globalShape, myShape_Blocks, myShape_InLeader;
    HitLayout layout_InLeader, layout_Blocks;
    HitTile_float globalTile, myTile_Blocks, myTile_InLeader;
    HitPattern pattern_InLeaderToBlocks, pattern_BlocksToInLeader;

    //I. Initialize the Hitmap library
    hit_comInit(&argc, &argv);
    
    arraySize = 50;

    if (arraySize < 1)
    {
        fprintf(stderr, "Error: arraySize can’t be zero or negative\n");
        return (-1);
    }

    //II. Define a (the, in this case) process topology as topPlain: use all processes
    globalTopology = hit_topology(plug_topPlain);

    //III. Define the shape of the global array that will be distributed
    globalShape = hit_shape(1, hit_sigStd(arraySize));

    Load_ratios ld;
    float w[] = {0.1,0.0002,0.3,0.01,0.1,0.5};
    ld.num_procs = 8;
    ld.ratios = w;

    //IV. Create the layout by applying the topology over a shape following a partitio
    layout_Blocks = hit_layout(plug_layDimBlocksWeighted, globalTopology, globalShape,0,ld);
    layout_InLeader = hit_layout(plug_layInLeader, globalTopology, globalShape);

    //V. Apply the shape of the ’virtual’ global array to the tile
    hit_tileDomainShape(&globalTile, float, globalShape);

    //VI. Ask the layout to get the shape of the local chunk
    myShape_Blocks = hit_layShape(layout_Blocks);
    myShape_InLeader = hit_layShape(layout_InLeader);

    /*if (hit_layImActive(layout_InLeader)) */
    
        HitSig hs1 = hit_shapeSig(myShape_Blocks , 0);
        printf("\n\n%s%d\n", "Proceso ", hit_Rank);
        printf("Dim 1 = %d:%d:%d\n", hs1.begin, hs1.end, hs1.stride);
        fflush(stdout);
        //printf("ret = %d\n", ret); */
    

    //VII. Create a hierarchy declaring the process’ local tile as part of the ’virtua
    hit_tileSelect(&myTile_Blocks,
                   &globalTile, myShape_Blocks);
    hit_tileSelect(&myTile_InLeader, &globalTile, myShape_InLeader);

    //VIII. Each process must allocate memory for its local tiles
    hit_tileAlloc(&myTile_Blocks);

    hit_tileAlloc(&myTile_InLeader);

    //IX. Create two sets of communication patterns: one to redistribute from the InLe
	pattern_InLeaderToBlocks = hit_patternLayRedistribute(layout_InLeader, layout_Blocks, &myTile_InLeader, &myTile_Blocks, HIT_FLOAT);
	pattern_BlocksToInLeader = hit_patternLayRedistribute(layout_Blocks, layout_InLeader, &myTile_Blocks, &myTile_InLeader, HIT_FLOAT);

    //X. Fill the array (for the InLeader layout) in the leader process with data
    if (hit_layImActive(layout_InLeader))
    {
        for (i = 0; i < hit_layDimCard(layout_InLeader, 0); i++)
        {
            hit_tileElemAt(myTile_InLeader, 1, i) = i;
        }
    }
    

//XI. Execute the data redistribution from the InLeader layout to the Blocks one
hit_patternDoOnce(pattern_InLeaderToBlocks);

//XII. Each process alters its own data within the Blocks tile
if (hit_layImActive(layout_Blocks))
{
        //printf("Rank active Blocks: %i, dimCard: %i\n", hit_Rank, hit_layD
        for (i = 0; i < hit_layDimCard(layout_Blocks, 0); i++)
            hit_tileElemAt(myTile_Blocks, 1, i) += 2;

}
//XIII. Execute the data redistribution from Blocks to InLeader layout
hit_patternDoOnce(pattern_BlocksToInLeader);

//XIV. Print the results from the leader process
if (hit_layImActive(layout_InLeader))
{
        //printf("Rank active inLeader: %i, dimCard: %i\n", hit_Rank, hit_la
        for (i = 0; i < hit_layDimCard(layout_InLeader, 0); i++)
            printf("%f ", hit_tileElemAt(myTile_InLeader, 1, i));
        printf("\n");
        fflush(stdout);
}

//XV. Free allocated tiles, layouts, and topologies
hit_tileFree(myTile_Blocks);
hit_tileFree(myTile_InLeader);
hit_layFree(layout_Blocks);
hit_layFree(layout_InLeader);
hit_topFree(globalTopology);

//XVI. Finalize the Hitmap library
hit_comFinalize();
return 0;
}
